package sun.management;

import java.lang.reflect.InvocationTargetException;
import java.lang.management.MonitorInfo;
import com.sun.management.VMOption;
import java.lang.management.MemoryNotificationInfo;
import java.lang.management.LockInfo;
import java.lang.management.ThreadInfo;
import java.lang.management.MemoryUsage;
import java.security.PrivilegedAction;
import java.security.PrivilegedActionException;
import java.security.AccessController;
import java.security.PrivilegedExceptionAction;
import java.lang.reflect.Method;
import java.util.HashMap;
import javax.management.openmbean.CompositeDataSupport;
import javax.management.openmbean.TabularDataSupport;
import javax.management.openmbean.TabularData;
import javax.management.openmbean.TabularType;
import javax.management.openmbean.CompositeType;
import java.util.ArrayList;
import java.util.Iterator;
import java.lang.reflect.Array;
import javax.management.openmbean.ArrayType;
import javax.management.openmbean.SimpleType;
import javax.management.openmbean.CompositeData;
import java.io.InvalidObjectException;
import javax.management.openmbean.OpenDataException;
import java.util.Map;
import java.util.List;
import java.lang.reflect.GenericArrayType;
import java.lang.reflect.ParameterizedType;
import javax.management.openmbean.OpenType;
import java.lang.reflect.Type;
import java.util.WeakHashMap;

public abstract class MappedMXBeanType
{
    private static final WeakHashMap<Type, MappedMXBeanType> convertedTypes;
    boolean isBasicType;
    OpenType<?> openType;
    Class<?> mappedTypeClass;
    private static final String KEY = "key";
    private static final String VALUE = "value";
    private static final String[] mapIndexNames;
    private static final String[] mapItemNames;
    private static final Class<?> COMPOSITE_DATA_CLASS;
    private static final OpenType<?> inProgress;
    private static final OpenType[] simpleTypes;
    
    public MappedMXBeanType() {
        this.isBasicType = false;
        this.openType = MappedMXBeanType.inProgress;
    }
    
    static synchronized MappedMXBeanType newMappedType(Type type) throws OpenDataException {
        MappedMXBeanType value = null;
        if ((type instanceof Class)) {
            Class<Object> clazz2 = (Class)type;
            if (clazz2.isEnum()) {
                value = new MappedMXBeanType.EnumMXBeanType(clazz2);
            }
            else if (clazz2.isArray()) {
                value = new MappedMXBeanType.ArrayMXBeanType(clazz2);
            }
            else {
                value = new MappedMXBeanType.CompositeDataMXBeanType(clazz2);
            }
        }
        else if ((type instanceof ParameterizedType)) {
            ParameterizedType parameterizedType = (ParameterizedType)type;
            Type rawType = parameterizedType.getRawType();
            if ((rawType instanceof Class)) {
                Class<Object> clazz = (Class)rawType;
                if (clazz == List.class) {
                    value = new ListMXBeanType(parameterizedType);
                }
                else if (clazz == Map.class) {
                    value = new MapMXBeanType(parameterizedType);
                }
            }
        }
        else if ((type instanceof GenericArrayType)) {
            value = new MappedMXBeanType.GenericArrayMXBeanType((GenericArrayType)type);
        }
        if (value == null) {
            throw new OpenDataException(type + " is not a supported MXBean type.");
        }
        MappedMXBeanType.convertedTypes.put(type, value);
        return value;
    }
    
    static synchronized MappedMXBeanType newBasicType(Class<?> key, OpenType<?> openType) throws OpenDataException {
        MappedMXBeanType.BasicMXBeanType value = new MappedMXBeanType.BasicMXBeanType(key, openType);
        MappedMXBeanType.convertedTypes.put(key, value);
        return value;
    }
    
    static synchronized MappedMXBeanType getMappedType(Type key) throws OpenDataException {
        MappedMXBeanType mappedType = (MappedMXBeanType)MappedMXBeanType.convertedTypes.get(key);
        if (mappedType == null) {
            mappedType = newMappedType(key);
        }
        if ((mappedType.getOpenType() instanceof MappedMXBeanType.InProgress)) {
            throw new OpenDataException("Recursive data structure");
        }
        return mappedType;
    }
    
    public static synchronized OpenType<?> toOpenType(Type type) throws OpenDataException {
        return getMappedType(type).getOpenType();
    }
    
    public static Object toJavaTypeData(Object o, Type type) throws OpenDataException, InvalidObjectException {
        if (o == null) {
            return null;
        }
        return getMappedType(type).toJavaTypeData(o);
    }
    
    public static Object toOpenTypeData(Object o, Type type) throws OpenDataException {
        if (o == null) {
            return null;
        }
        return getMappedType(type).toOpenTypeData(o);
    }
    
    OpenType<?> getOpenType() {
        return this.openType;
    }
    
    boolean isBasicType() {
        return this.isBasicType;
    }
    
    String getTypeName() {
        return this.getMappedTypeClass().getName();
    }
    
    Class<?> getMappedTypeClass() {
        return this.mappedTypeClass;
    }
    
    abstract Type getJavaType();
    
    abstract String getName();
    
    abstract Object toOpenTypeData(Object p0) throws OpenDataException;
    
    abstract Object toJavaTypeData(Object p0) throws OpenDataException, InvalidObjectException;
    
    private static String decapitalize(String s) {
        if (s == null || s.length() == 0) {
            return s;
        }
        if (s.length() > 1 && Character.isUpperCase(s.charAt(1)) && Character.isUpperCase(s.charAt(0))) {
            return s;
        }
        char[] charArray = s.toCharArray();
        charArray[0] = Character.toLowerCase(charArray[0]);
        return new String(charArray);
    }
    
    static String access$300(String s) {
        return decapitalize(s);
    }
    
    static {
        convertedTypes = new WeakHashMap();
        mapIndexNames = new String[] { "key" };
        mapItemNames = new String[] { "key", "value" };
        COMPOSITE_DATA_CLASS = CompositeData.class;
        MappedMXBeanType.InProgress inProgress2;
        try {
            inProgress2 = new MappedMXBeanType.InProgress();
        }
        catch (OpenDataException detailMessage) {
            throw new AssertionError(detailMessage);
        }
        inProgress = inProgress2;
        simpleTypes = new OpenType[] { SimpleType.BIGDECIMAL, SimpleType.BIGINTEGER, SimpleType.BOOLEAN, SimpleType.BYTE, SimpleType.CHARACTER, SimpleType.DATE, SimpleType.DOUBLE, SimpleType.FLOAT, SimpleType.INTEGER, SimpleType.LONG, SimpleType.OBJECTNAME, SimpleType.SHORT, SimpleType.STRING, SimpleType.VOID };
        try {
            for (int i = 0; i < MappedMXBeanType.simpleTypes.length; ++i) {
                OpenType<Object> openType = MappedMXBeanType.simpleTypes[i];
                Class<Object> forName;
                try {
                    forName = Class.forName(openType.getClassName(), false, MappedMXBeanType.class.getClassLoader());
                    newBasicType(forName, openType);
                }
                catch (ClassNotFoundException detailMessage2) {
                    throw new AssertionError(detailMessage2);
                }
                catch (OpenDataException detailMessage3) {
                    throw new AssertionError(detailMessage3);
                }
                if (forName.getName().startsWith("java.lang.")) {
                    try {
                        newBasicType((Class)forName.getField("TYPE").get(null), openType);
                    }
                    catch (NoSuchFieldException ex) {}
                    catch (IllegalAccessException detailMessage4) {
                        throw new AssertionError(detailMessage4);
                    }
                }
            }
        }
        catch (OpenDataException detailMessage5) {
            throw new AssertionError(detailMessage5);
        }
    }
    
    static class BasicMXBeanType extends MappedMXBeanType
    {
        final Class<?> basicType;
        
        BasicMXBeanType(Class<?> clazz, OpenType<?> openType) {
            this.basicType = clazz;
            this.openType = openType;
            this.mappedTypeClass = clazz;
            this.isBasicType = true;
        }
        
        @Override
        Type getJavaType() {
            return this.basicType;
        }
        
        @Override
        String getName() {
            return this.basicType.getName();
        }
        
        @Override
        Object toOpenTypeData(Object o) throws OpenDataException {
            return o;
        }
        
        @Override
        Object toJavaTypeData(Object o) throws OpenDataException, InvalidObjectException {
            return o;
        }
    }
    
    static class EnumMXBeanType extends MappedMXBeanType
    {
        final Class enumClass;
        
        EnumMXBeanType(Class<?> enumClass) {
            this.enumClass = enumClass;
            this.openType = SimpleType.STRING;
            this.mappedTypeClass = String.class;
        }
        
        @Override
        Type getJavaType() {
            return this.enumClass;
        }
        
        @Override
        String getName() {
            return this.enumClass.getName();
        }
        
        @Override
        Object toOpenTypeData(Object o) throws OpenDataException {
            return ((Enum)o).name();
        }
        
        @Override
        Object toJavaTypeData(Object o) throws OpenDataException, InvalidObjectException {
            try {
                return Enum.valueOf(this.enumClass, (String)o);
            }
            catch (IllegalArgumentException cause) {
                InvalidObjectException ex = new InvalidObjectException("Enum constant named " + (String)o + " is missing");
                ex.initCause(cause);
                throw ex;
            }
        }
    }
    
    static class ArrayMXBeanType extends MappedMXBeanType
    {
        final Class<?> arrayClass;
        protected MappedMXBeanType componentType;
        protected MappedMXBeanType baseElementType;
        
        ArrayMXBeanType(Class<?> componentType) throws OpenDataException {
            this.arrayClass = componentType;
            this.componentType = MappedMXBeanType.getMappedType(componentType.getComponentType());
            StringBuilder sb = new StringBuilder();
            int dimension;
            for (, dimension = 0; componentType.isArray(); componentType = componentType.getComponentType(), ++dimension) {
                sb.append('[');
            }
            this.baseElementType = MappedMXBeanType.getMappedType(componentType);
            if (componentType.isPrimitive()) {
                sb = new StringBuilder(componentType.getName());
            }
            else {
                sb.append("L" + this.baseElementType.getTypeName() + ";");
            }
            try {
                this.mappedTypeClass = Class.forName(sb.toString());
            }
            catch (ClassNotFoundException cause) {
                OpenDataException o = new OpenDataException("Cannot obtain array class");
                o.initCause(cause);
                throw o;
            }
            this.openType = new ArrayType(dimension, this.baseElementType.getOpenType());
        }
        
        protected ArrayMXBeanType() {
            this.arrayClass = null;
        }
        
        @Override
        Type getJavaType() {
            return this.arrayClass;
        }
        
        @Override
        String getName() {
            return this.arrayClass.getName();
        }
        
        @Override
        Object toOpenTypeData(Object o) throws OpenDataException {
            if (this.baseElementType.isBasicType()) {
                return o;
            }
            Object[] array = (Object[])(Object[])o;
            Object[] array2 = (Object[])(Object[])Array.newInstance(this.componentType.getMappedTypeClass(), array.length);
            int n = 0;
            for (int length = array.length, i = 0; i < length; ++n, ++i) {
                Object o2 = array[i];
                if (o2 == null) {
                    array2[n] = null;
                }
                else {
                    array2[n] = this.componentType.toOpenTypeData(o2);
                }
            }
            return array2;
        }
        
        @Override
        Object toJavaTypeData(Object o) throws OpenDataException, InvalidObjectException {
            if (this.baseElementType.isBasicType()) {
                return o;
            }
            Object[] array = (Object[])(Object[])o;
            Object[] array2 = (Object[])(Object[])Array.newInstance((Class)this.componentType.getJavaType(), array.length);
            int n = 0;
            for (int length = array.length, i = 0; i < length; ++n, ++i) {
                Object o2 = array[i];
                if (o2 == null) {
                    array2[n] = null;
                }
                else {
                    array2[n] = this.componentType.toJavaTypeData(o2);
                }
            }
            return array2;
        }
    }
    
    static class GenericArrayMXBeanType extends MappedMXBeanType.ArrayMXBeanType
    {
        final GenericArrayType gtype;
        
        GenericArrayMXBeanType(GenericArrayType genericComponentType) throws OpenDataException {
            this.gtype = genericComponentType;
            this.componentType = MappedMXBeanType.getMappedType(genericComponentType.getGenericComponentType());
            StringBuilder sb = new StringBuilder();
            int dimension;
            for (, dimension = 0; genericComponentType instanceof GenericArrayType; genericComponentType = ((GenericArrayType)genericComponentType).getGenericComponentType(), ++dimension) {
                sb.append('[');
            }
            this.baseElementType = MappedMXBeanType.getMappedType(genericComponentType);
            if (genericComponentType instanceof Class && ((Class)genericComponentType).isPrimitive()) {
                sb = new StringBuilder(genericComponentType.toString());
            }
            else {
                sb.append("L" + this.baseElementType.getTypeName() + ";");
            }
            try {
                this.mappedTypeClass = Class.forName(sb.toString());
            }
            catch (ClassNotFoundException cause) {
                OpenDataException o = new OpenDataException("Cannot obtain array class");
                o.initCause(cause);
                throw o;
            }
            this.openType = new ArrayType(dimension, this.baseElementType.getOpenType());
        }
        
        @Override
        Type getJavaType() {
            return this.gtype;
        }
        
        @Override
        String getName() {
            return this.gtype.toString();
        }
    }
    
    static class ListMXBeanType extends MappedMXBeanType
    {
        final ParameterizedType javaType;
        final MappedMXBeanType paramType;
        final String typeName;
        
        ListMXBeanType(ParameterizedType obj) throws OpenDataException {
            this.javaType = obj;
            Type[] actualTypeArguments = obj.getActualTypeArguments();
            assert actualTypeArguments.length == 1;
            if (!(actualTypeArguments[0] instanceof Class)) {
                throw new OpenDataException("Element Type for " + obj + " not supported");
            }
            Class<Object> clazz = (Class)actualTypeArguments[0];
            if (clazz.isArray()) {
                throw new OpenDataException("Element Type for " + obj + " not supported");
            }
            this.paramType = MappedMXBeanType.getMappedType(clazz);
            this.typeName = "List<" + this.paramType.getName() + ">";
            try {
                this.mappedTypeClass = Class.forName("[L" + this.paramType.getTypeName() + ";");
            }
            catch (ClassNotFoundException cause) {
                OpenDataException ex = new OpenDataException("Array class not found");
                ex.initCause(cause);
                throw ex;
            }
            this.openType = new ArrayType(1, this.paramType.getOpenType());
            return;
        }
        
        @Override
        Type getJavaType() {
            return this.javaType;
        }
        
        @Override
        String getName() {
            return this.typeName;
        }
        
        @Override
        Object toOpenTypeData(Object o) throws OpenDataException {
            List<Object> list = (List)o;
            Object[] array = (Object[])(Object[])Array.newInstance(this.paramType.getMappedTypeClass(), list.size());
            int n = 0;
            Iterator<Object> iterator = list.iterator();
            while (iterator.hasNext()) {
                array[n++] = this.paramType.toOpenTypeData(iterator.next());
            }
            return array;
        }
        
        @Override
        Object toJavaTypeData(Object o) throws OpenDataException, InvalidObjectException {
            Object[] array = (Object[])(Object[])o;
            ArrayList list = new ArrayList(array.length);
            for (int length = array.length, i = 0; i < length; ++i) {
                list.add(this.paramType.toJavaTypeData(array[i]));
            }
            return list;
        }
    }
    
    static class MapMXBeanType extends MappedMXBeanType
    {
        final ParameterizedType javaType;
        final MappedMXBeanType keyType;
        final MappedMXBeanType valueType;
        final String typeName;
        
        MapMXBeanType(ParameterizedType javaType) throws OpenDataException {
            this.javaType = javaType;
            Type[] actualTypeArguments = javaType.getActualTypeArguments();
            assert actualTypeArguments.length == 2;
            this.keyType = MappedMXBeanType.getMappedType(actualTypeArguments[0]);
            this.valueType = MappedMXBeanType.getMappedType(actualTypeArguments[1]);
            this.typeName = "Map<" + this.keyType.getName() + "," + this.valueType.getName() + ">";
            this.openType = new TabularType(this.typeName, this.typeName, new CompositeType(this.typeName, this.typeName, MappedMXBeanType.mapItemNames, MappedMXBeanType.mapItemNames, new OpenType[] { this.keyType.getOpenType(), this.valueType.getOpenType() }), MappedMXBeanType.mapIndexNames);
            this.mappedTypeClass = TabularData.class;
            return;
        }
        
        @Override
        Type getJavaType() {
            return this.javaType;
        }
        
        @Override
        String getName() {
            return this.typeName;
        }
        
        @Override
        Object toOpenTypeData(Object o) throws OpenDataException {
            Map<Object, Object> map = (Map)o;
            TabularType tabularType = (TabularType)this.openType;
            TabularDataSupport tabularDataSupport = new TabularDataSupport(tabularType);
            CompositeType rowType = tabularType.getRowType();
            for (Map.Entry entry : map.entrySet()) {
                tabularDataSupport.put(new CompositeDataSupport(rowType, MappedMXBeanType.mapItemNames, new Object[] { this.keyType.toOpenTypeData(entry.getKey()), this.valueType.toOpenTypeData(entry.getValue()) }));
            }
            return tabularDataSupport;
        }
        
        @Override
        Object toJavaTypeData(Object o) throws OpenDataException, InvalidObjectException {
            TabularData tabularData = (TabularData)o;
            HashMap<Object, Object> hashMap = new HashMap();
            for (CompositeData compositeData : tabularData.values()) {
                hashMap.put(this.keyType.toJavaTypeData(compositeData.get("key")), this.valueType.toJavaTypeData(compositeData.get("value")));
            }
            return hashMap;
        }
    }
    
    static class CompositeDataMXBeanType extends MappedMXBeanType
    {
        final Class<?> javaClass;
        final boolean isCompositeData;
        Method fromMethod;
        
        CompositeDataMXBeanType(Class<?> javaClass) throws OpenDataException {
            this.fromMethod = null;
            this.javaClass = javaClass;
            this.mappedTypeClass = MappedMXBeanType.COMPOSITE_DATA_CLASS;
            try {
                this.fromMethod = (Method)AccessController.doPrivileged(new PrivilegedExceptionAction<Method>(this) {
                    @Override
                    public Method run() throws NoSuchMethodException {
                        return javaClass.getMethod("from", new Class[] { MappedMXBeanType.COMPOSITE_DATA_CLASS });
                    }
                });
            }
            catch (PrivilegedActionException ex) {}
            if (MappedMXBeanType.COMPOSITE_DATA_CLASS.isAssignableFrom(javaClass)) {
                this.isCompositeData = true;
                this.openType = null;
            }
            else {
                this.isCompositeData = false;
                Method[] array = (Method[])AccessController.doPrivileged(new PrivilegedAction<Method[]>(this) {
                    @Override
                    public Method[] run() {
                        return javaClass.getMethods();
                    }
                });
                ArrayList<String> list = new ArrayList();
                ArrayList<OpenType<?>> list2 = new ArrayList();
                for (int i = 0; i < array.length; ++i) {
                    Method method = array[i];
                    String name = method.getName();
                    Type genericReturnType = method.getGenericReturnType();
                    String s;
                    if (name.startsWith("get")) {
                        s = name.substring(3);
                    }
                    else if (name.startsWith("is") && genericReturnType instanceof Class && (Class)genericReturnType == Boolean.TYPE) {
                        s = name.substring(2);
                    }
                    else {
                        continue;
                    }
                    if (!s.equals("") && method.getParameterTypes().length <= 0 && genericReturnType != Void.TYPE && !s.equals("Class")) {
                        list.add(MappedMXBeanType.access$300(s));
                        list2.add(MappedMXBeanType.toOpenType(genericReturnType));
                    }
                }
                String[] array2 = (String[])list.toArray(new String[0]);
                this.openType = new CompositeType(javaClass.getName(), javaClass.getName(), array2, array2, (OpenType[])list2.toArray(new OpenType[0]));
            }
        }
        
        @Override
        Type getJavaType() {
            return this.javaClass;
        }
        
        @Override
        String getName() {
            return this.javaClass.getName();
        }
        
        @Override
        Object toOpenTypeData(Object o) throws OpenDataException {
            if ((o instanceof MemoryUsage)) {
                return MemoryUsageCompositeData.toCompositeData((MemoryUsage)o);
            }
            if ((o instanceof ThreadInfo)) {
                return ThreadInfoCompositeData.toCompositeData((ThreadInfo)o);
            }
            if ((o instanceof LockInfo)) {
                if ((o instanceof MonitorInfo)) {
                    return MonitorInfoCompositeData.toCompositeData((MonitorInfo)o);
                }
                return LockInfoCompositeData.toCompositeData((LockInfo)o);
            }
            else {
                if ((o instanceof MemoryNotificationInfo)) {
                    return MemoryNotifInfoCompositeData.toCompositeData((MemoryNotificationInfo)o);
                }
                if ((o instanceof VMOption)) {
                    return VMOptionCompositeData.toCompositeData((VMOption)o);
                }
                if (this.isCompositeData) {
                    CompositeData compositeData = (CompositeData)o;
                    CompositeType compositeType = compositeData.getCompositeType();
                    String[] itemNames = (String[])compositeType.keySet().toArray(new String[0]);
                    return new CompositeDataSupport(compositeType, itemNames, compositeData.getAll(itemNames));
                }
                throw new OpenDataException(this.javaClass.getName() + " is not supported for platform MXBeans");
            }
        }
        
        @Override
        Object toJavaTypeData(Object o) throws OpenDataException, InvalidObjectException {
            if (this.fromMethod == null) {
                throw new AssertionError("Does not support data conversion");
            }
            try {
                return this.fromMethod.invoke(null, new Object[] { o });
            }
            catch (IllegalAccessException detailMessage) {
                throw new AssertionError(detailMessage);
            }
            catch (InvocationTargetException cause) {
                OpenDataException ex = new OpenDataException("Failed to invoke " + this.fromMethod.getName() + " to convert CompositeData  to " + this.javaClass.getName());
                ex.initCause(cause);
                throw ex;
            }
        }
    }
    
    private static class InProgress extends OpenType
    {
        private static final String description = "Marker to detect recursive type use -- internal use only!";
        private static final long serialVersionUID = -3413063475064374490L;
        
        InProgress() throws OpenDataException {
            super("java.lang.String", "java.lang.String", "Marker to detect recursive type use -- internal use only!");
        }
        
        @Override
        public String toString() {
            return "Marker to detect recursive type use -- internal use only!";
        }
        
        @Override
        public int hashCode() {
            return 0;
        }
        
        @Override
        public boolean equals(Object o) {
            return false;
        }
        
        @Override
        public boolean isValue(Object o) {
            return false;
        }
    }
}
